#Remove this file
from string import strip, find, join
listType = type([])

FILTER_MATCHES=0
FILTER_MISSES=1
FILTER_BOTH=2


class FilterPresent:
    def __init__(self, attribute):
	self.attribute = attribute

    def __call__(self, objs, ReturnType=FILTER_MATCHES):
	if type(objs) is not listType:
	    raise 'AttributeError', 'Filters must be passed a list of objects'
	
	if not -1 < ReturnType < 3:
	    raise 'AttributeError', 'ReturnType must be a value between 0 and 2'
	    
	matches = []
	misses  = []
	for obj in objs:
	    if hasattr(obj, self.attribute): 
		matches.append(obj)
	    else:
		misses.append(obj)
	
	if ReturnType == FILTER_MATCHES:
	    return matches
	elif ReturnType == FILTER_MISSES:
	    return misses
	else:
	    return (matches, misses)

    def filter2sql(self, negate=0):
	if negate == 0: 
	    return "%s NOT NULL" % self.attribute
	else: # Return the opisite of this query
	    return "%s IS NULL" % self.attribute

	

class FilterEquality:
    def __init__(self, attribute, value):
	self.attribute = attribute
	self.value = value

    def __call__(self, objs, ReturnType=FILTER_MATCHES):
	if type(objs) is not listType:
	    raise 'AttributeError', 'Filters must be passed a list of objects'

	if not -1 < ReturnType < 3:
	    raise 'AttributeError', 'ReturnType must be a value between 0 and 2'

	    
	matches = []
	misses  = []
	for obj in objs:
	    if hasattr(obj, self.attribute) and \
	       getattr(obj, self.attribute) == self.value: 
		    matches.append(obj)
	    else:
		misses.append(obj)

	if ReturnType == FILTER_MATCHES:
	    return matches
	elif ReturnType == FILTER_MISSES:
	    return misses
	else:
	    return (matches, misses)

    def filter2sql(self, negate=0):
	if negate == 0:
            # TODO: Fix this hack
	    #return "%s = %s" % (self.attribute, self.value)
            return "%s = %s" % (self.attribute, self.value)
	else: # Return the opisite of this query
	    return "%s <> %s" % (self.attribute, self.value)

class FilterRe:
    def __init__(self, attribute, value):
	self.attribute = attribute
	self.value = value

    def __call__(self, objs, ReturnType=FILTER_MATCHES):
	if type(objs) is not listType:
	    raise 'AttributeError', 'Filters must be passed a list of objects'

	if not -1 < ReturnType < 3:
	    raise 'AttributeError', 'ReturnType must be a value between 0 and 2'

	    
	matches = []
	misses  = []
	for obj in objs:
	    if hasattr(obj, self.attribute) and \
	       getattr(obj, self.attribute) == self.value: 
		    matches.append(obj)
	    else:
		misses.append(obj)

	if ReturnType == FILTER_MATCHES:
	    return matches
	elif ReturnType == FILTER_MISSES:
	    return misses
	else:
	    return (matches, misses)
		    

class FilterListAnd:
    def __init__(self, filters):
	self.filters = filters

    
    def __call__(self, objs, ReturnType=FILTER_MATCHES):
	if type(objs) is not listType:
	    raise 'AttributeError', 'Filters must be passed a list of objects'

	if not -1 < ReturnType < 3:
	    raise 'AttributeError', 'ReturnType must be a value between 0 and 2'

	    
	matches = objs
	misses  = []

	for f in self.filters:
	    matches, additional_misses = apply(f,(matches, FILTER_BOTH))
	    misses = misses + additional_misses

	if ReturnType == FILTER_MATCHES:
	    return matches
	elif ReturnType == FILTER_MISSES:
	    return misses
	else:
	    return (matches, misses)

    def filter2sql(self, negate=0):
	filters = self.filters
	results = []
	for f in filters:
	    results.append('(%s)' % f.filter2sql(negate))
	
	if negate == 0:
	    return join(results, '\nAND\n')
	else: # The oposite of an AND is an OR
	    return join(results, '\nOR\n')


class FilterListOr:
    def __init__(self, filters):
	self.filters = filters

    
    def __call__(self, objs, ReturnType=FILTER_MATCHES):
	if type(objs) is not listType:
	    raise 'AttributeError', 'Filters must be passed a list of objects'

	if not -1 < ReturnType < 3:
	    raise 'AttributeError', 'ReturnType must be a value between 0 and 2'

	    
	matches = []
	misses  = objs 

	for f in self.filters:
	    additional_matches, misses = apply(f,(misses, FILTER_BOTH))
	    matches = matches + additional_matches

	if ReturnType == FILTER_MATCHES:
	    return matches
	elif ReturnType == FILTER_MISSES:
	    return misses
	else:
	    return (matches, misses)

    def filter2sql(self, negate=0):
	filters = self.filters
	results = []
	for f in filters:
	    results.append('(%s)' % f.filter2sql(negate))
	
	if negate == 0:
	    return join(results, '\nOR\n')
	else: # The oposite of an OR is an AND
	    return join(results, '\nAND\n')


class FilterListNot:
    def __init__(self, filters):
	self.filters = filters

    
    def __call__(self, objs, ReturnType=FILTER_MATCHES):
	if type(objs) is not listType:
	    raise 'AttributeError', 'Filters must be passed a list of objects'

	if not -1 < ReturnType < 3:
	    raise 'AttributeError', 'ReturnType must be a value between 0 and 2'

	    
	matches = objs
	misses  = []

	for f in self.filters:
	    additional_misses, matches = apply(f,(matches, FILTER_BOTH))
	    misses  = misses  + additional_misses
	    #matches = matches + additional_misses

	if ReturnType == FILTER_MATCHES:
	    return matches
	elif ReturnType == FILTER_MISSES:
	    return misses
	else:
	    return (matches, misses)

    def filter2sql(self, negate=0):
	filters = self.filters
	results = []
	for f in filters:
	    results.append('(%s)' % f.filter2sql(not negate))
	    
	
	if negate == 0:
	    return join(results, '\nAND\n')
	else: # The oposite of an OR is an AND
	    return join(results, '\nOR\n')


def str2filter(str):
    str = strip(str)
    if str == '':
	raise 'ValueError', 'Filter must be a non-empty string.'
    
    if str[0:1] == '(':
	end = findClosingParen(str)
	str = strip(str[1:end])
	c = str[0:1]

	if c == '&':
	    str = strip(str[1:])
	    return FilterListAnd(str2filterlist(str))
	elif c == '|':
	    str = strip(str[1:])
	    return FilterListOr(str2filterlist(str))
	elif c == '!':
	    str = strip(str[1:])
	    return FilterListNot(str2filterlist(str))

	else:
	    return str2simplefilter(str)
    else:
	return str2simplefilter(str)
    

def str2simplefilter(str):
    CompIndex = find(str, '=')
    if CompIndex == -1:
	raise 'ValueError', 'Filter must be in the format of <attr> <comp> <value>.'

    CompType = str[CompIndex-1]
    
    if CompType == '<':
	raise 'NotImplemented', 'Less Than Filter has not been implemented yet.'
    elif CompType == '>':
	raise 'NotImplemented', 'Greater Than Filter has not been implemented yet.'
    if CompType == '~':
	raise 'NotImplemented', 'Regular Expresion Filter has not been implemented yet.'
    else: # Equality or Precense
	attribute = strip(str[:CompIndex])
	value = strip(str[CompIndex+1:])
	
	if value == '*':
	    return FilterPresent(attribute)
	else:
	    return FilterEquality(attribute, value)
	



def str2filterlist(s):
    filters = []

    while(s):
	end = findClosingParen(s)
	filters.append(str2filter(s))
	s = strip(s[end+1:])
    
    return filters

def findClosingParen(str):
    balance = 0
    escape = 0
    index = 0

    c = str[index:index+1]
    while c:

	if not escape:
	    if c == '(':
		balance = balance + 1
	    elif c == ')':
		balance = balance - 1
	if balance == 0:
	    return index

	if (c == '\\') and not escape:
	    escape = 1
	else: escape = 0

	index = index + 1
	c = str[index:index+1]

    raise 'ValueError', 'Parenthesis Mismatch'


## class O:
##     def __init__(self, x, y=None):
## 	self.x = str(x)
	
## 	if y is not None:
## 	    self.y = str(y)

## data = [O(1,2), O(3), O(3,1)]
## # return a list of all objects that have a y argument and an x argument = 1
## s = 'x="Frank"'
## f = str2filter(s)

## print """SELECT *
## FROM bung
## WHERE
## %s""" % f.filter2sql()
