# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
__doc__='''Preform a search
''' 
__rcs_id__='$Id: SearchTag.py,v 1.2 1999/12/06 04:33:40 sroberts Exp $'
__version__='$Revision: 1.2 $'[11:-2]

from DocumentTemplate.DT_Util import ParseError, parse_params, name_param, str, render_blocks
from DocumentTemplate.DT_String import String
from DocumentTemplate.DT_In import InClass

class Search:
    """Search catalogs"""

    name='search'
    blockContinuations='results','else'

    def __init__(self, blocks):
	if blocks[1][0] != 'results': raise ParseError, (
	    'There must be one and only one results block.', 'search')

        tname, args, section = blocks[0]
        args=parse_params(args, catalog='', distinct=1)
	
	if not args.has_key(''): args[''] = 'queryProducts'


	catalog, expr=name_param(args,'catalog',1)
        self.catalog  = catalog

	self.args = args
	getarg=self.getarg_
 
	self.distinct = getarg('distinct', 0)

	self.blocks = section.blocks

	# The results block inherits from the IN tag.
	# This tag does every thing that the IN tag does
	# except it always calls the same expresion which
	# returns the results from the search.
	# Yeah I know it's hacky but this way I don't need
	# to rewrite the whole stinkin in class. =)
	
	tname, args, section = blocks[1]
	blocks[1] = tname, ('expr="%s(querystr)" ' % self.catalog)  + args, section
	self.results = InClass(blocks[1:])

    def getarg_(self, name, defval):
        if self.args.has_key(name):
            return self.args[name]
        else:
            return defval

    def render(self, md):
	# Render the query and add it to the list of name spaces to
	# be searched
	querystr = render_blocks(self.blocks, md)
	md._push({'querystr': querystr})

	#import pdb
	#pdb.set_trace()

	i = self.results

	if i.batch: return i.renderwb(md)
        else: return i.renderwob(md)

    __call__ = render


    
    
# Install tag globally. This really sucks because this tag
# should only be available to objects under a shop.
String.commands['search'] = Search

##         if blocks[-1][0]=='else':
##             tname, args, section = blocks[-1]
##             del blocks[-1]
##             args=parse_params(args, name='')
##             if args:
##                 ename,expr=name_param(args,'else',1)
##                 if ename != name:
##                     raise ParseError, ('name in else does not match if', 'in')
##             elses=section.blocks
##         else: elses=None

##         for tname, args, section in blocks[1:]:
##             if tname=='else':
##                 raise ParseError, (
##                     'more than one else tag for a single if tag', 'in')
##             args=parse_params(args, name='', expr='')
##             name,expr=name_param(args,'elif',1)
##             if expr is None: cond=name
##             else: cond=expr.eval
##             sections.append(cond)
##             sections.append(section.blocks)

##         if elses is not None: sections.append(elses)

##        self.simple_form=tuple(sections)








## class Unless:
##     name='unless'
##     blockContinuations=()

##     def __init__(self, blocks):
##         tname, args, section = blocks[0]
##         args=parse_params(args, name='', expr='')
##         name,expr=name_param(args,'unless',1)
##         if expr is None: cond=name
##         else: cond=expr.eval
##         self.simple_form=(cond,None,section.blocks)

