from Storage import Storage
from Products.ZSQLMethods.SQL import SQL
from string import join, split, lower
from Filters import str2filter

from Order import Order
from OrderEntry import OrderEntry

from Category import Category
from Product import Product


class SQLStorage(Storage):
    def _init(self, table_name, fields, primary_key, wrapper_class, conn_id):
	self.table_name    = table_name
	self.fields        = fields
	self.primary_key   = primary_key
	self.conn_id       = str(conn_id)
	self.wrapper_class = wrapper_class

	self.sqlAddObject = SQL('sqlAddObject', '', self.conn_id,
				join(fields.keys(), '\n'),
				self._makeSqlAddStr())

	self.sqlUpdateObject = SQL('sqlUpdateObject', '', self.conn_id,
				join(fields.keys(), '\n'),
				self._makeSqlUpdateStr())

	self.sqlDeleteObjects = SQL('sqlDeleteObjects', '', self.conn_id,
				  'ids', self._makeSqlDeleteStr())

	self.sqlQueryObjects = SQL('sqlQueryObjects', '', self.conn_id,
				  'query_str', self._makeSqlQueryStr())
	
    def _makeSqlAddStr(self):
	fields = self.fields.keys()
	sqlstr = "INSERT into  %s\n" % self.table_name +\
		 "(%s)" % join(fields, ',') +\
		 "VALUES(\n" 
		 
	index = 0
	for field in fields:
	    fields[index] = "<dtml-sqlvar %s type=%s>" % (field, 
							  self.fields[field])
	    index = index + 1 
	
	sqlstr = sqlstr + join(fields, ',\n') + ')'

	return sqlstr



    def _makeSqlUpdateStr(self):
	fields = self.fields.keys()
	sqlstr = "UPDATE %s\n" % self.table_name +\
		 "SET\n" 
		 
	index = 0
	for field in fields:
	    fields[index] = "<dtml-sqltest %s type=%s>" % (field, 
							  self.fields[field])
	    index = index + 1 
	
	sqlstr = sqlstr + join(fields, ',\n') 

	sqlstr = sqlstr + "\nWHERE\n" +\
		 "%s = <dtml-sqlvar %s type=%s>" %\
		 (self.primary_key, 
		  self.primary_key, 
		  self.fields[self.primary_key])
	

	return sqlstr

    def _makeSqlDeleteStr(self):
	sqlstr = "DELETE FROM %s\n" % self.table_name + \
		 "WHERE\n<dtml-sqltest ids column=%s type=%s multiple>" %\
		 (self.primary_key, 
		  self.fields[self.primary_key])

	
	return sqlstr


    def _makeSqlQueryStr(self):
	sqlstr = "SELECT * \nFROM %s\n" % self.table_name + \
		 "WHERE\n<dtml-var query_str>"
	
	return sqlstr




    def addObject(self, object):
	self.sqlAddObject(object.__dict__)

    def updateObject(self, object):
	self.sqlUpdateObject(object.__dict__)

    def deleteObjects(self, object_ids):
	self.sqlDeleteObjects(ids=object_ids)
	

    def queryObjects(self, qrystr):
	'Find items in the storage that match the query'

 	query_str = str2filter(qrystr).filter2sql()
	results = self.sqlQueryObjects(query_str=query_str)
	
	return ResultWrapper(results, self.wrapper_class, self)

class SQLObject:

    def __init__(self, rr):
	attrs = rr.__record_schema__.keys()

	for attr in attrs:
	    setattr(self, lower(attr), rr[attr])

class ResultWrapper: 

    """This class is used to wrap result objects with an appropriate
    class. It's an optimization."""

    def __init__(self, r, wrapper_class, parent):
	self.results = r
	self.wrapper_class = wrapper_class
	self.parent = parent

    def __getattr__(self, attr):
	return getattr(self.results, attr)
    
    def __getitem__(self, index):
	
	# TODO: This function could be further optimized by
	# implementing a cache of objects that have already been
	# wrapped.
        
	object = self.results[index]
	return self.wrapper_class(object).__of__(self.parent)




class CategoryStorage(SQLStorage):
    "This string is only here at them moment so I can look at it through the web. "
    def __init__(self, conn_id):
	SQLStorage._init(self, 'categorystorage', # Table Name
			 {'id': 'string',      # Field Names
			  'label':'string', 
			  'category_id': 'string'
			  },
			 'id',                 # Primary Key
			 sqlCategory,          # Wrapper Class
 			 conn_id	       # DB Connection
			 )


class ProductStorage(SQLStorage):
    
    """This string is only here at them moment so I can look at it
    through the web."""

    def __init__(self, conn_id):
	SQLStorage._init(self, 'productstorage', # Table Name
			 {'sku': 'string',       # Field Definitions
			  'description': 'string',
			  'price':'float', 
			  'category_id':'string'
			 },
			 'sku',                  # Primary Key
			 sqlProduct,             # Wrapper Class
			 conn_id		 # DB Connection
			 )

class sqlCategory(SQLObject, Category):
    "SQL Category object"

    meta_type = "SQL Category"

    def addCategory(self, category):
	"Add a sub-category to this category"
	# Set the parent of this category
	#category.category_id = self.id
	self.CategoryStorage.addObject(category)

    def getCategory(self, id):
	query_str = "(&(id='%s')(category_id='%s'))" % (id, self.getUID())
	r = self.CategoryStorage.queryObjects(query_str)

	if len(r) > 0:
	    return r[0]
	else:
	    return None

    def updateCategory(self, category):
	self.CategoryStorage.updateObject(category)

    def deleteCategories(self, ids, REQUEST=None):
        "Delete selected categories"
	self.CategoryStorage.deleteObjects(ids)
        if REQUEST is not None:
            return self.manage_products(self, REQUEST)

    
    def listCategories(self):
	"List subcategories"
        
	query_str = "category_id='%s'" % self.getUID()
	return self.CategoryStorage.queryObjects(query_str)


    def addProduct(self, product):
	"Add a product to this category"
	# Set the category of this product
	#product.category_id = self.id
	self.ProductStorage.addObject(product)

    def getProduct(self, id):
	# query_str = "(&(sku='%s')(category_id='%s'))" % (id, self.getUID())
	# TODO: Fix me.
	query_str = "sku='%s'" % id
	
	r = self.ProductStorage.queryObjects(query_str)

	if len(r) > 0:
	    return r[0]
	else:
	    return None

    def updateProduct(self, product):
	self.ProductStorage.updateObject(product)

    def deleteProducts(self, object_ids):
	self.ProductStorage.deleteObjects(object_ids)

    
    def listProducts(self):
	"List subcategories"
	query_str = "category_id='%s'" % self.getUID()
        results = self.ProductStorage.queryObjects(query_str)
	return results
	

class sqlProduct(SQLObject, Product):
    "SQL Product"
    pass





class OrderStorage(SQLStorage):
    "This string is only here at them moment so I can look at it through the web."
    
    def __init__(self, conn_id):
	SQLStorage._init(self, 'orderstorage', # Table Name
			 {'id': 'string',    # Field Names
			  'date':'string', 
			  'status':'string', 
			  'customer':'string', 
			  'payment_method':'string', 
			  'delivery_method':'string', 
			  'billto_fullname':'string', 
			  'billto_address1':'string', 
			  'billto_address2':'string', 
			  'billto_city':'string', 
			  'billto_state':'string', 
			  'billto_zip':'string', 
			  'billto_country':'string', 
			  'shipto_fullname':'string', 
			  'shipto_address1':'string', 
			  'shipto_address2':'string', 
			  'shipto_city':'string', 
			  'shipto_state':'string', 
			  'shipto_zip':'string', 
			  'shipto_country':'string', 
			  'cc_num':'string', 
			  'cc_exp':'string', 
			  },
			 'id',                # Primary Key
			 sqlOrder,            # Wrapper Class
                         conn_id
			 )
	self.OrderEntryStorage = OrderEntryStorage(conn_id)


class OrderEntryStorage(SQLStorage):
    
    "This string is only here at them moment so I can look at it through the web."

    def __init__(self, conn_id):
	SQLStorage._init(self, 'orderentrystorage', # Table Name
			 {'id': 'string',    # Field Names
			  'sku': 'string',
			  'price':'float', 
			  'quantity':'string', 
			  'order_id':'string'
			 },
			 'id',               # Primary Key
			 sqlOrderEntry,      # Wrapper Class
                         conn_id
			 )

    

class sqlOrder(SQLObject, Order):

    def addOrderEntry(self, order_entry):
	order_entry.order_id = self.id
	self.OrderEntryStorage.addObject(order_entry)

    def getOrderEntry(self, sku):
	query_str = "(&(sku='%s')(order_id='%s'))" % (sku, self.id)
	r = self.OrderEntryStorage.queryObjects(query_str)

	if len(r) > 0:
	    return r[0]
	else:
	    return None

    def updateOrderEntry(self, order_entry):
	self.OrderEntryStorage.updateObject(order_entry)

    def deleteOrderEntries(self, object_ids):
	self.OrderEntryStorage.deleteObjects(object_ids)

    
    def listOrderEntries(self):
	query_str = "order_id='%s'" % self.id
	return self.OrderEntryStorage.queryObjects(query_str)
	
    def getSubTotal(self):
        subtotal = 0.0
        entries = self.listOrderEntries()
        
        for entry in entries:
            subtotal = subtotal + (int(entry.quantity) * float(entry.price))

        return subtotal

class sqlOrderEntry(SQLObject, OrderEntry):

    def getAmount(self):
        return int(self.quantity) * float(self.price)
	
    
